﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Base;
using Tuple = Infragistics.Olap.Data.Base.Tuple;

namespace PivotGridUsingFlatPivotTable.DataSource
{
	/*
	 * Is overriding DataSourceBase the correct way to start this process of hooking my server
	 * data source to the pivot grid?
	 */
	public class FlatPivotDataSource : DataSourceBase
	{
		private readonly IFlatPivotServerDataSource serverDataSource;

		public FlatPivotDataSource(IFlatPivotServerDataSource serverDataSource)
		{
			this.serverDataSource = serverDataSource;
		}

		/*
		 * This conversion is very confusing to me, as its not completely clear (without reading
		 * complete OLAP specs) how the tuples, members and levels need to be organised in the axes
		 * to display correctly in the grid. I imagine its easy if you're connecting to an existing
		 * OLAP source (which the model presumably fits perfectly), but for custom sources such as
		 * mine it is less than perfectly clear.
		 */
		protected virtual IResult BuildResultFromFlatPivotTable(FlatPivotTable flatPivotTable)
		{
			var columnAxis = BuildResultAxis(flatPivotTable.XAxis);
			var rowAxis = BuildResultAxis(flatPivotTable.YAxis);
			var cells = BuildResultCells(flatPivotTable);
			return new Result(rowAxis, columnAxis, cells);
		}

		protected virtual IResultAxis BuildResultAxis(FlatPivotAxis flatPivotAxis)
		{
			var tuples = new List<ITuple>();
			ProcessFlatPivotAxis(flatPivotAxis, tuples, null, 0, String.Empty);
			return new ResultAxis(tuples, true /* what does this really do? */);
		}

		/*
		 * A bleak attempt at converting my axis into an IResultAxis...
		 */
		protected virtual void ProcessFlatPivotAxis(FlatPivotAxis flatPivotAxis, IList<ITuple> tuples, IMember parentMember, int levelDepth, string uniqueNamePrefix)
		{
			// Crude - I know, and all guesswork
			var name = flatPivotAxis.Header.ToString();
			var uniqueName = (uniqueNamePrefix != String.Empty ? uniqueNamePrefix + "_" : String.Empty) + flatPivotAxis.Header;
			var parentUniqueName = parentMember != null ? parentMember.UniqueName : String.Empty;
			var childCount = flatPivotAxis.Children != null ? flatPivotAxis.Children.Count : 0;

			// Each "node" in my axis represents a tuple from what what I can gauge. Its not entirely
			// clear how the member I specify should be related to other members, but I'm guessing the 
			// level thats implied in my hierarchy comes into play.

			// Creating the member here...its a somewhat odd list of constructor parameters, in that its 
			// not clear what they should all really be in the grand scheme of things.
			var member = new Member(
				name,
				uniqueName,
				name,
				parentUniqueName,
				String.Empty,			// This should probably be set...but to what?
				levelDepth,				// Guessing that I incrememt from 0 based on my hierarchy depth
				null,					// Again, this might need setting, but its not clear how I define levels
				childCount,
				true,					// How should these booleans really be set?
				true,
				true);

			tuples.Add(new Tuple(new OlapElementCollection<IMember>(new List<IMember> { member })));
			
			// Repeat recursively for all children?
			if (flatPivotAxis.Children != null)
			{
				foreach (var child in flatPivotAxis.Children)
				{
					ProcessFlatPivotAxis(child.Value, tuples, member, levelDepth + 1, uniqueName);
				}
			}
		}

		/*
		 * This bit seems relatively simple, just copy values one-to-one from my source
		 * to the ICell array.
		 */
		protected virtual ICell[,] BuildResultCells(FlatPivotTable flatPivotTable)
		{
			var cells = new ICell[flatPivotTable.YAxis.Width, flatPivotTable.XAxis.Width];
			for (var y = 0; y < cells.GetLength(0); y++)
			{
				for (var x = 0; x < cells.GetLength(1); x++)
				{
					var value = flatPivotTable.Cells[y][x];
					cells[y, x] = new Cell(value, value.ToString());
				}
			}
			return cells;
		}

		/*
		 * Not sure whether I should be doing this sort of operation here, also not clear to me
		 * how I instruct the data source to refresh data. Not sure how to invoke it (if that's 
		 * what I'm supposed to be doing)
		 */
		protected override void BuildDataInternal(BackgroundWorker backgroundWorker, DoWorkEventArgs e)
		{
			// Get data from the "server"
			var flatPivotTable = serverDataSource.LoadData();

			// Is this supposed to work?
			e.Result = BuildResultFromFlatPivotTable(flatPivotTable);
		}

		public void RefreshDataHackStyle()
		{
			// Get data from the "server"
			var flatPivotTable = serverDataSource.LoadData();

			// Try setting result directly using my converted flat pivot table
			Result = BuildResultFromFlatPivotTable(flatPivotTable);

			// Fire result changed - not sure on parameters here (apart from obvious ones re. errors)
			OnResultChanged(new ResultChangedEventArgs(Result, null, null, false, null));
		}
	}
}
